 -- to access configuration control chip, use BAR1
-- BAR1 occupies 16 byte space
-- 0x0 write address
-- 0x4 write/read data
-- 0x8 resets virtex chip
-- 0xC write reconfigure commmand
-- All accesses have to be 32-bit word. Byte enable signals are assumed to be all active.
-- flash memory address register has address 0x0 and is write-only. Only data bits 19-0 are used
-- flash mmeory data register has address 0x4. write access uses bits 7-0 for flash data.
-- read access returns
-- bits 31-14 all '0'
-- bit 13 is '1' indicates that configuration started from 0x40000 
-- bit 12 pci3 done signal  
-- bit 11 pci2 done signal  
-- bit 10 pci1 done signal  
-- bit 9 virtex done signal  
-- bit 8 virtex init signal
-- bit 7-0 are flash data
-- address register does not auto-increment
-- write to data register automatically starts the execution of the command
-- for flash memory operation, please refer to AM29F080B data sheet
--
-- Following any wirte/read operation, flash controller automatically reads the flash for later access
-- It is important to notice that any read access always gets the data resulted from the last read/write access.
--  
-- Any access takes 2+ microsecond
-- to accomplish. Any new access before it is ready again will result in a retry. 
--
-- to start a reconfiguration, write 0xa5 to data register at address 0xC, this starts the reconfiguration
--
-- flash memory contents
-- 0x00000 - pci3
-- 0x10000 - pci1
-- 0x20000 - pci2
-- 0x30000 - virtex
--
-- stc_ctl_o(2) = '0' valid data
-- stc_ctl_o(3) = '1' sdram write ready
-- stc_ctl_o(4) = '1' end of l3 event
-- stc_ctl_o(5) = '1' insert read wait state
-- stc_ctl_o(6) = '1' abort
-- stc_ctl_i(2) = '1' resets read fifo word count
-- stc_ctl_i(3) = '1' reset virtex DLL
-- 11/20/03 brst_ovfl added
LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;
USE ieee.std_logic_arith.all;

ENTITY pci3_if IS

	PORT
	(
-- pci megafunction interface
-- inputs        
               clock:		in std_logic;  -- PCI clock input, rising edge active
               rstn:		in std_logic;  --- PCI reset input 
               lt_framen:	in std_logic;  --- PCI interface needs local access
               lt_ackn:		in std_logic;  --- PCI inetrface is ready for r/w data
               lt_dxfrn:	in std_logic;  --- last target data transfer successful
               lt_tsr:		in std_logic_vector(11 downto 0);  --- target transaction status
               l_dato:		in std_logic_vector(31 downto 0);  --- local data input
               l_adro:		in std_logic_vector(31 downto 0);  --- local address input
               l_cmdo:		in std_logic_vector(3 downto 0);  --- local command input
-- outputs
               lt_rdyn:		out std_logic;  --- local target ready for r/w data
               lt_discn:	out std_logic;  --- local target disconnect output
               lt_abortn:	out std_logic;  --- local target abort output
	           l_adi:		out std_logic_vector(31 downto 0);  --- local data output
-- stc interface    
    	       stc_data:	inout std_logic_vector(31 downto 0);
    		   stc_ctl_o:	in std_logic_vector(6 downto 0);
     		   stc_ctl_i:	out std_logic_vector(4 downto 0);
		       stc_clk:		in std_logic;
-- configuration controls
-- inputs
		      pci_sp_i:	in	std_logic_vector(3 downto 0);
-- outputs
		      pci_sp_o:	out	std_logic_vector(7 downto 0);
              led: out std_logic_vector(3 downto 0)
    );
	
END pci3_if;
ARCHITECTURE my_arch OF pci3_if IS
COMPONENT lpm_ram_dp
   GENERIC ( LPM_WIDTH: POSITIVE :=34;
             LPM_WIDTHAD: POSITIVE := 4;
             LPM_NUMWORDS: NATURAL := 16;
             LPM_TYPE: STRING := "LPM_RAM_DP";
             LPM_INDATA: STRING := "REGISTERED";
             LPM_OUTDATA: STRING := "REGISTERED"; -- 11/25/03 somehow this does not work??
             LPM_RDADDRESS_CONTROL: STRING := "UNREGISTERED"; -- 11/25/03 somehow this does not work??
--             LPM_OUTDATA: STRING := "UNREGISTERED";
--             LPM_RDADDRESS_CONTROL: STRING := "REGISTERED";
             LPM_WRADDRESS_CONTROL: STRING := "REGISTERED";
             LPM_FILE: STRING := "UNUSED";
             LPM_HINT: STRING := "UNUSED"

           );
   PORT    ( rdaddress, wraddress: IN STD_LOGIC_VECTOR(LPM_WIDTHAD-1 DOWNTO 0);
             rdclock, wrclock: IN STD_LOGIC := '0';
             rden, rdclken, wrclken: IN STD_LOGIC := '1';
             wren: IN STD_LOGIC; 
             data: IN STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0);
             q: OUT STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0));
END COMPONENT;
COMPONENT lpm_counter
   GENERIC (LPM_WIDTH: POSITIVE := 12;
      LPM_MODULUS: NATURAL := 0;
      LPM_DIRECTION: STRING := "UNUSED";
      LPM_AVALUE: STRING := "UNUSED";
      LPM_SVALUE: STRING := "UNUSED";
      LPM_PVALUE: STRING := "UNUSED";
      LPM_TYPE: STRING := "LPM_COUNTER";
      LPM_HINT: STRING := "UNUSED");
   PORT (data: IN STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0) := (OTHERS => '0');
      clock: IN STD_LOGIC;
      cin: IN STD_LOGIC := '0';
      clk_en: IN STD_LOGIC := '1';
      cnt_en: IN STD_LOGIC := '1';
      updown: IN STD_LOGIC := '1';
      sload: IN STD_LOGIC := '0';
      sset: IN STD_LOGIC := '0';
      sclr: IN STD_LOGIC := '0';
      aload: IN STD_LOGIC := '0';
      aset: IN STD_LOGIC := '0';
      aclr: IN STD_LOGIC := '0';
      cout: OUT STD_LOGIC := '0';
      q: OUT STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0));
END COMPONENT;
component gray4
  port (clk: in std_logic;
         ec: in std_logic;
        clrn: in std_logic;
         q: out std_logic_vector(3 downto 0));
end component;
signal write_ready_dl: std_logic;
-- configuration interface
constant tc: integer := 31;
signal conf_tmp0, conf_tmp1: std_logic_vector(20 downto 0);
signal conf_sync: std_logic_vector(2 downto 0);
signal conf_data, conf_tgl, conf_write, conf_shift, conf_din: std_logic;
signal conf_clk: std_logic;
-- state machine for configuration interface
signal en_count: std_logic;
signal count: integer range 0 to tc;
signal single_read, sel_bar1: std_logic;
-- stc interface signals
signal rfifo_din,rfifo_dout : std_logic_vector(33 downto 0);
signal rfifo_q : std_logic_vector(31 downto 0);
signal ec_rfifo_q,rfifo_dav : std_logic;
signal ld_addr,wfifo_dav,wfifo_we,wfifo_re: std_logic;
signal rfifo_re,rfifo_we,rfifo_empty,stc_wr_rdy,stc_rd_rdy: std_logic;
signal conf_ready: std_logic;
signal wfifo_wa, wfifo_ra, sync_wwa, sync2_wwa: std_logic_vector(3 downto 0);
signal sync_wra, sync2_wra: std_logic_vector(1 downto 0);
signal wfifo_din, wfifo_dout : std_logic_vector(35 downto 0);
signal stc_ctl_oq: std_logic_vector(6 downto 0);
signal eoc_sync: std_logic_vector(2 downto 0);
signal inh_write,toggle_r_eoc,got_eoc: std_logic;
signal wait_w_done: std_logic_vector(2 downto 0);
signal sdram_busy: std_logic;
signal stc_data_i,stc_data_o,t_stc_data: std_logic_vector(31 downto 0);
signal rfifo_ra, rfifo_wa : std_logic_vector(3 downto 0);
signal wr_ad_sync,wr_ad_syncp: std_logic_vector(3 downto 0);
signal rst_stcn,stc_ready,to_flag,dis_connect,discn,data_sent: std_logic;
signal stc_init: unsigned(9 downto 0);
signal burst_cnt: unsigned(11 downto 0);
signal burst_cntp: std_logic_vector(11 downto 0);
signal timeout: unsigned(5 downto 0);
signal gnd,vcc,lt_rdy: std_logic;
signal wait_state,first_l3_data: std_logic;
signal conf_tgl_dl,conf_busy: std_logic;
signal en_count_sync: std_logic_vector(2 downto 0);
-- l3 data trailer
signal trailer: std_logic;
signal clr_brst_cntr: std_logic;
signal en_brst_cntr: std_logic;
signal brst_ovfl: std_logic;
signal brst_ovfl_r: std_logic;
BEGIN
gnd <= '0';
vcc <= '1';
--stc_ctl_i(6 downto 5) <= "00";
stc_ctl_i(3) <= not rst_stcn;
--stc_ctl_i(3) <= '0';
--stc_ctl_i(2) <= lt_tsr(0);
led <= "ZZZZ";
--
-- instantiate write fifo's dpram, which is mainly used for synchronization purpose only. It is assumed that the
-- dcc side has double the clock rate so it will never get full.
wfifo: lpm_ram_dp
   generic map(LPM_WIDTH => 36)
   port map(
           rdaddress => wfifo_ra,
           wraddress => wfifo_wa,
           rdclock => stc_clk,
           wrclock => clock,
           rdclken => wfifo_re,-- 11/25/03 somehow this does not work??
--           rden => wfifo_re,
           wren => wfifo_we,
           data => wfifo_din,
           q => wfifo_dout);
wfifo_din(35) <= not lt_framen and lt_tsr(0);
wfifo_din(34) <= trailer;
wfifo_din(33) <= l_cmdo(0) or ld_addr;
wfifo_din(32) <= ld_addr;
wfifo_din(31 downto 0) <= l_dato when ld_addr = '0' else l_cmdo(0) & lt_tsr(9) & first_l3_data & "000000000" & l_adro(19 downto 0);
-- reading total word counter must also generate event_sent in the Xilinx pci3_if so that new word count could be read
first_l3_data <= '1' when (l_adro(17 downto 2) = x"8000" or l_adro(17 downto 2) = x"a009") and l_cmdo(0) = '0' else '0';
wfifo_we <= ((not lt_framen and lt_tsr(0)) xor not ld_addr) or (not lt_framen and lt_tsr(0) and not lt_dxfrn);
process(clock,rstn)
variable a: std_logic_vector(3 downto 0);
begin
  a := wfifo_wa(3 downto 2) & sync2_wra;
  if(rstn = '0')then
    sync_wra <= "00";
    sync2_wra <= "00";
    stc_wr_rdy <= '0';
    eoc_sync <= "000";
    inh_write <= '0';
  elsif(clock'event and clock = '1')then
    sync_wra <= wfifo_ra(3 downto 2);
    sync2_wra <= sync_wra;
    case a is
      when "0001" | "0111" | "1110" | "1000" => stc_wr_rdy <= lt_dxfrn and lt_tsr(0) and l_cmdo(0) and not inh_write;
      when others => stc_wr_rdy <= lt_tsr(0) and l_cmdo(0) and not inh_write;
    end case;
    eoc_sync <= eoc_sync(1 downto 0) & toggle_r_eoc;
    inh_write <= (inh_write and not(eoc_sync(2) xor eoc_sync(1))) or (not(not lt_framen and lt_tsr(0)) and not ld_addr);
  end if;
end process;
-------------------------------------------
process(stc_clk,rstn)
begin
  if(rstn = '0')then
    sdram_busy <= '0';
    wait_w_done <= "000";
    got_eoc <= '0';
    toggle_r_eoc <= '0';
  elsif(stc_clk'event and stc_clk = '1')then
    if(wfifo_dav = '1' and wfifo_dout(35) = '1' and wfifo_dout(33 downto 32) = "10")then
      sdram_busy <= '1';
      wait_w_done <= "111";
    else
-- stc_ctl_oq(3) is sdram write ready signal
      if(stc_ctl_oq(3) = '1' and wait_w_done(2) = '0')then
        sdram_busy <= '0';
      end if;
      wait_w_done(2) <= wait_w_done(2) and (wait_w_done(1) or wait_w_done(0));
      wait_w_done(1) <= wait_w_done(2) and (wait_w_done(1) xor not wait_w_done(0));
      wait_w_done(0) <= wait_w_done(2) and not wait_w_done(0);
    end if;
-- end of pci cycle detection    
    got_eoc <= (wfifo_dav and not wfifo_dout(35)) or (got_eoc and sdram_busy);
    toggle_r_eoc <= (got_eoc and not sdram_busy) xor toggle_r_eoc;
  end if;
end process;
--process(clock)
--begin
--  if(clock'event and clock = '1')then
--    if(lt_framen = '0' and lt_tsr(0) = '1' and ld_addr = '1')then
--      burst_cnt <= "000000000000";
--    elsif(lt_tsr(0) = '1' and lt_dxfrn = '0')then
--      burst_cnt <= burst_cnt + 1;
--    end if;
--  end if;
--end process;
process(clock,stc_ready)
begin
  if(stc_ready = '0')then
    ld_addr <= '0';
  elsif(clock'event and clock = '1')then
    ld_addr <= not(not lt_framen and lt_tsr(0));
  end if;
end process;
i_wfifo_wa: gray4 port map(clk => clock, clrn => stc_ready, ec => wfifo_we, q => wfifo_wa);
i_wfifo_ra: gray4 port map(clk => stc_clk, clrn => stc_ready, ec => wfifo_re, q => wfifo_ra);
-- synchronize wfifo_wa and generate read enable
process(stc_clk,stc_ready)
begin
  if(stc_ready = '0')then
    wfifo_dav <= '0';
    sync_wwa <= "0000";
    sync2_wwa <= "0000";
  elsif(stc_clk'event and stc_clk = '1')then
    wfifo_dav <= wfifo_re;
    sync_wwa <= wfifo_wa;
    sync2_wwa <= sync_wwa;
  end if;
end process;
wfifo_re <= '1' when sync2_wwa /= wfifo_ra and (wfifo_dout(35) = '1' or sdram_busy = '0') and (wfifo_dav = '0' or wfifo_dout(33) = '0') else '0';
-- stc_data
process(stc_clk)
begin
  if(stc_clk'event and stc_clk = '1')then
    stc_data_o <= wfifo_dout(31 downto 0);
  end if;
end process;
g_stc_data: for i in stc_data'range generate
  stc_data(i) <= stc_data_o(i) when t_stc_data(i) = '0' else 'Z';
end generate;
-- stc_ctl
process(stc_clk,stc_ready)
begin
  if(stc_ready = '0')then
    stc_ctl_i(2 downto 0) <= "000";
    stc_ctl_i(4) <= '0';
    t_stc_data <= (others => '1');
  elsif(stc_clk'event and stc_clk = '1')then
-- read address is NOP
-- data/address
    stc_ctl_i(0) <= wfifo_dav and wfifo_dout(35) and not wfifo_dout(32);
-- write/read
    stc_ctl_i(1) <= wfifo_dav and wfifo_dout(35) and wfifo_dout(33);
    if(wfifo_dav = '1')then
      stc_ctl_i(2) <= wfifo_dout(35);
    end if;
    stc_ctl_i(4) <= wfifo_dout(34);
    for i in stc_data'range loop
      t_stc_data(i) <= not (wfifo_dav and wfifo_dout(35) and wfifo_dout(33));
    end loop; 
  end if;
end process;
-------------------------------------------
process(stc_clk,stc_ready)
begin
  if(stc_ready = '0')then
    stc_ctl_oq <= "1111111";
  elsif(stc_clk'event and stc_clk = '1')then
    stc_ctl_oq <= stc_ctl_o;
  end if;
end process;
process(stc_clk)
begin
  if(stc_clk'event and stc_clk = '1')then
    stc_data_i <= stc_data;
  end if;
end process;
-- stc_ctl_oq(2) input data valid, low active
rfifo_we <= not stc_ctl_oq(2); 
----------------------------------------------------------------------------------------------------------
-- instantiate read fifo's dpram
-- stc_ctl_oq(5) end of l3 event
-- stc_ctl_oq(4) insert read wait state
rfifo_din <= stc_ctl_oq(5 downto 4) & stc_data_i;
rfifo: lpm_ram_dp
   port map(
           rdaddress => rfifo_ra,
           wraddress => rfifo_wa,
           rdclock => clock,
           wrclock => stc_clk,
           rdclken => rfifo_re,-- 11/25/03 somehow this does not work??
--           rden => rfifo_re,
           wren => rfifo_we,
           data => rfifo_din,
           q => rfifo_dout);
--------- r_fifo read/write address
i_rfifo_wa: gray4 port map(clk => stc_clk, clrn => lt_tsr(0), ec => rfifo_we, q => rfifo_wa);
i_rfifo_ra: gray4 port map(clk => clock, clrn => lt_tsr(0), ec => rfifo_re, q => rfifo_ra);
rfifo_re <= not rfifo_empty and (not rfifo_dav or ec_rfifo_q);
rfifo_empty <= '1' when rfifo_ra = wr_ad_sync else '0';
ec_rfifo_q <= (rfifo_dav or trailer) and (not stc_rd_rdy or not lt_dxfrn);
process(clock)
begin
  if(clock'event and clock = '1')then
    if(ec_rfifo_q = '1')then
      trailer <= rfifo_dout(33);
    end if;
  end if;
end process;
process(clock)
begin
  if(clock'event and clock = '1')then
    if(ec_rfifo_q = '1')then
	  rfifo_q <= rfifo_dout(31 downto 0);
    end if;
  end if;
end process;
process(clock,lt_tsr)
begin
  if(lt_tsr(0) = '0')then
    rfifo_dav <= '0';
    stc_rd_rdy <= '0';
    wr_ad_syncp <= "0000";
    wr_ad_sync <= "0000";
  elsif(clock'event and clock = '1')then
    rfifo_dav <= not rfifo_empty or (rfifo_dav and not ec_rfifo_q);
    stc_rd_rdy <= rfifo_dav or (stc_rd_rdy and lt_dxfrn);
-- synchronize fifo write address
    wr_ad_syncp <= rfifo_wa;
    wr_ad_sync <= wr_ad_syncp;
  end if;
end process;
--g_burst: for i in burst_cnt'range generate
--  burst_cntp(i) <= burst_cnt(i);
--end generate;
l_adi <= rfifo_q when lt_tsr(0) = '1' else wfifo_ra & burst_cntp & "00" & conf_tmp1(13 downto 0);
-- configuration interface
process(clock,rstn)
begin
  if(rstn = '0')then
    conf_tmp0 <= (others => '0');
    conf_tgl <= '0';
    conf_data <= '0';
    conf_write <= '0';
    pci_sp_o(5) <= '0';
    rst_stcn <= '0';
    stc_ready <= '0';
    stc_init <= (others => '0');
  elsif(clock'event and clock = '1')then
-- pci_sp_i(3) is virtex2 done signal. Reset is delayed until done is high
    if(stc_ready = '0' and pci_sp_i(3) = '1')then
       stc_init <= stc_init + 1;
    else
       stc_init <= (others => '0');
    end if;
    if(rst_stcn = '0')then
      stc_ready <= '0';
    elsif(stc_init(9) = '1')then
      stc_ready <= '1';
    end if;
    if(lt_dxfrn = '0')then
      conf_tgl <= (lt_tsr(1) and not(l_adro(3) and not l_adro(2))) xor conf_tgl;
    end if;
    if(pci_sp_i(3) = '0')then
      rst_stcn <= '0';
    elsif(lt_tsr(1) = '1' and lt_dxfrn = '0' and l_adro(3 downto 2) = "10" and l_dato = x"5555aaaa" and l_cmdo(0) = '1')then
      rst_stcn <= '0';
    else
      rst_stcn <= '1';
    end if;
    if(lt_tsr(1) = '1' and lt_dxfrn = '0')then
-- this is for starting reconfiguration
      pci_sp_o(5) <= l_adro(3);
      conf_data <= l_adro(2);
      conf_write <= l_cmdo(0);
      conf_tmp0 <= l_dato(20 downto 0);
    end if;
  end if;
end process;
conf_clk <= pci_sp_i(1);
process(conf_clk)
begin
-- using falling edge to receive data from xc9572
  if(conf_clk'event and conf_clk = '0')then
    conf_din <= pci_sp_i(0);
  end if;
-- using rising edge for internal clock and sending data to xc9572
  if(conf_clk'event and conf_clk = '1')then
-- synchronize rw_conf to conf_clk
    conf_sync(0) <= conf_tgl;
    conf_sync(1) <= conf_sync(0);
    conf_sync(2) <= conf_sync(1) xor conf_sync(0);
-- don't care if it is a read or a write, always load conf_tmp1 with conf_tmp0 when conf is accessed
    if(conf_sync(2) = '1')then
      conf_tmp1 <= conf_tmp0;
    elsif(conf_shift = '1')then
      conf_tmp1(0) <= conf_din;
      for i in 1 to 19 loop
        conf_tmp1(i) <= conf_tmp1(i-1);
      end loop;
    end if;
-- select shift register output bit 
    if(conf_data = '1')then
      pci_sp_o(4) <= conf_tmp1(7);
    else
      pci_sp_o(4) <= conf_tmp1(19);
    end if;
  end if;
end process;
-- state machine of configuration interface
process(conf_clk)
begin
  if(conf_clk'event and conf_clk = '1')then
    if(conf_sync(2) = '1')then
      en_count <= '1';
    elsif(count = tc)then
      en_count <= '0';
    end if;
    if(en_count = '0')then
      count <= 0;
    else
      count <= count + 1;
    end if;
    conf_shift <= en_count;
-- load address signal
    case count is
      when 0 to 19 =>
        pci_sp_o(0) <= en_count and not conf_data;
      when others =>
        pci_sp_o(0) <= '0';
    end case;
-- read flash signal
    case count is
      when 0 to 19 =>
        pci_sp_o(1) <= en_count and not conf_data and conf_tmp1(20);
      when 22 =>
        pci_sp_o(1) <= '1';
      when others =>
        pci_sp_o(1) <= '0';
    end case;
-- write flash signal
    case count is
      when 7 to 11 =>
        pci_sp_o(2) <= conf_data and conf_write;
      when others =>
        pci_sp_o(2) <= '0';
    end case;
-- shift data input signal
    case count is
-- write address
      when 0 to 7 | 10 to 12=>
        pci_sp_o(3) <= conf_data and conf_write and en_count;
      when 17 to 29 =>
        pci_sp_o(3) <= '1';
      when others =>
        pci_sp_o(3) <= '0';
    end case;
  end if;
end process;
process(clock)
begin
  if(clock'event and clock = '1')then
-- abort l3 data readout during SCL INIT. pci_sp_i(3) is virtex done signal
    lt_abortn <= not (stc_ctl_oq(6) and pci_sp_i(3));
-- no burst access if lt_tsr(1) = '1'
    sel_bar1 <= lt_tsr(1) and not lt_framen;
-- single read has different lt_discn timing
--    single_read <= not l_cmdo(0) and lt_tsr(1) and not en_count;
  end if;
end process;
brst_cntr: lpm_counter
  port map (
            clock => clock,
            cnt_en => en_brst_cntr,
            sclr => clr_brst_cntr,
            q => burst_cntp,
            cout => brst_ovfl);
en_brst_cntr <= not lt_dxfrn and lt_tsr(0);
clr_brst_cntr <= not lt_framen and lt_tsr(0) and ld_addr;
--brst_ovfl_r <= burst_cntp(4);
process(clock)
begin
  if(clock'event and clock = '1')then
    brst_ovfl_r <= brst_ovfl;
  end if;
end process;
--
lt_discn <= (not sel_bar1 or (not l_cmdo(0) and lt_dxfrn and conf_ready)) and (not(trailer or brst_ovfl_r) or lt_dxfrn);
--lt_discn <= lt_dxfrn when single_read = '1' else discon and discn;
-- if disconnect is requested without any data sent, it should be an abort
pci_sp_o(7 downto 6) <= "00";
lt_rdyn <= not lt_rdy;
--lt_rdy <= conf_ready or (stc_rd_rdy and ((rfifo_dav and not rfifo_dout(32)) or lt_dxfrn)) or stc_wr_rdy or timeout(5);
lt_rdy <= conf_ready or (stc_rd_rdy and ((rfifo_dav and not rfifo_dout(32)) or lt_dxfrn)) or stc_wr_rdy;
process(clock,rstn)
begin
  if (rstn = '0')then
    conf_tgl_dl <= '0';
    en_count_sync <= "000";
    conf_busy <= '0';
    conf_ready <= '0';
  elsif(clock'event and clock = '1')then
    conf_tgl_dl <= conf_tgl;
    en_count_sync(0) <= en_count;
    en_count_sync(1) <= en_count_sync(0);
    en_count_sync(2) <= en_count_sync(1) and not en_count_sync(0);
    conf_busy <= (conf_tgl xor conf_tgl_dl) or (conf_busy and not en_count_sync(2));
    conf_ready <= lt_tsr(1) and not lt_framen and ((not conf_busy and not sel_bar1) or (conf_ready and sel_bar1));
  end if;
end process;
-- lt_rdyn lock out protection
--process(clock,rstn)
--begin
--  if(rstn = '0')then
--    timeout <= "000000";
--    to_flag <= '0';
--  elsif(clock'event and clock = '1')then
--    if(lt_tsr(1) = '1' and l_cmdo(0) = '0' and lt_dxfrn = '0')then
--      to_flag <= '0';
--    end if;
--    if(timeout(5) = '1')then
--      to_flag <= '1';
--    end if;
--    if(lt_tsr(1 downto 0) /= "00" and lt_rdy = '0')then
--      timeout <= timeout + 1;
--    else
--      timeout <= "000000";
--    end if;
--  end if;
--end process;
END my_arch;
-----------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.all;
entity gray4 is
  port( clk: in std_logic;
         ec: in std_logic;
       clrn: in std_logic;
          q: out std_logic_vector(3 downto 0));
end gray4;
architecture gray4_arch of gray4 is
signal q_i: std_logic_vector(3 downto 0);
begin
q <= q_i;
process(clk,clrn)
begin
  if(clrn = '0')then
    q_i <= "0000";
  elsif(clk'event and clk = '1')then
    if(ec = '1')then
      q_i(3) <= (q_i(3) and (q_i(1) or q_i(0))) or (q_i(2) and not q_i(1) and not q_i(0));
      q_i(2) <= (q_i(2) and (not q_i(1) or q_i(0))) or (not q_i(3) and q_i(1) and not q_i(0));
      q_i(1) <= (q_i(1) and not q_i(0)) or ((q_i(3) xnor q_i(2)) and q_i(0));
      q_i(0) <= not(q_i(3) xor q_i(2) xor q_i(1));
    end if;
  end if;
end process;
end gray4_arch;
